title: Software is a medium of setbacks, but a medium’s limitations don’t define the artist
url: https://www.baldurbjarnason.com/2024/software-is-a-medium-of-setbacks/
hash_url: d133365d1dec62b9ad7ab4c911db863a
archive_date: 2024-02-21
og_image: https://www.baldurbjarnason.com/favicon-96x96.png
description: Much of the time, progress in software development comes coupled with a matching regression.“We’ve made it better.”(Whispered) “But we’ve also made it worse."The evolution of Apple’s computing devices makes for a decent example.
favicon: https://www.baldurbjarnason.com/dark-bird.svg
language: en_US

<img src="https://www.baldurbjarnason.com/img/hveragerdi-dither-experiment.png" alt="A view of the waterfall in Varmá. The picture is dithered and blocky. Reminiscent of how the original mac computers rendered pictures.">
<p>Much of the time, progress in software development comes coupled with a matching regression.</p>
<p>“We’ve made it better.”</p>
<p>(Whispered) <em>“But we’ve also made it worse."</em></p>
<p>The evolution of Apple’s computing devices makes for a decent example.</p>
<p>The original Mac? Huge leap in the usability and effectiveness of the desktop computer. Also didn’t come with a programming language and, initially, software for it had to be developed on an entirely different, much more expensive system (the Apple Lisa).</p>
<p>Over time that situation improved with Hypercard and various forms of scripting and automation, as well as a decent set of tools for making software with Pascal and later C++.</p>
<p>MacOS X arrived, and we got the power of the unix-compatible ecosystem finally coupled with a usable desktop environment.</p>
<p>We also lost many of the scripting capabilities that were unique to Mac “Classic”, but that was sorta-kinda offset by the wealth of new command-line tools and built-in support for popular scripting languages such as Python.</p>
<p>Sucked if you were all-in on AppleScript, but great for the rest of us.</p>
<p>Swift is a modern programming language designed for the development of complex but reliable systems. But when it was first launched it both was substantially less capable than Objective-C, and it sucked all the air out of the room for the entire Objective-C ecosystem. Apple had to spend years both maintaining an old language, one it was clearly trying to get rid of, and evolving a new language to the point where it could replace the old. The innovation came with a built-in setback that took years of effort to overcome.</p>
<p>Not that Swift is a true replacement for Objective-C. Its goals are clearly <em>very</em> different from Objective-C which leaned heavily into late binding and runtime dynamism. Swift leans the other way, focusing instead on correctness and performance. The differences between the two languages that most end-users or even developers notice are more down to the framework and environment and not the languages, per se, but there is a <em>flavour</em> difference – that vague sensation that both spoken and programming languages leave you with, an added dimension of meaning that hangs around it like a meal’s smell in a kitchen.</p>
<p>Progress is an ongoing act of myth-making. Most of the time the true story is a mixed one. Things are lost. Others gained. A happy ending is when you come out ahead. A story full of setbacks is a tragedy.</p>
<p>It feels like we’re experiencing more setbacks than progress in software and tech at the moment.</p>
<p>Lay-offs are having a noticeable effect on the software the tech industry is shipping. I know I’m finding buggy updates to be more common than they were a couple of years ago. <em>(Setback.)</em> The decline in search engines over only a few months is as undeniable as a sack hammers swung at your face. <em>(Setback.)</em></p>
<p>And that brings up the other setback. In studies LLM-aided coding is <a href="https://visualstudiomagazine.com/Articles/2024/01/25/copilot-research.aspx">not</a> <a href="https://arxiv.org/abs/2211.03622">having</a> a <a href="https://arxiv.org/abs/2108.09293">great</a> <a href="https://arxiv.org/abs/2208.09727">effect</a> on the <a href="https://softwarecrisis.dev/letters/ai-and-software-quality/">quality</a> or <a href="https://arstechnica.com/information-technology/2024/01/ai-poisoning-could-turn-open-models-into-destructive-sleeper-agents-says-anthropic/">safety</a> of the code. <em>(Setback.)</em> Sure, they’re more “productive” in that they’re writing more lines of code, but the industry discovered decades ago that measuring coding productivity this way is a very bad idea as it inherently promotes poor coding. <em>(Setback.)</em></p>
<p>Defenders say that in actual practice nobody actually commits LLM-generated code directly so this is all just scare-mongering, except – because we, as an industry, is so utterly foolish as to commit completely to a Microsoft-owned development environment through and through <em>(Setback)</em> – Microsoft can track the code generated by Copilot, through Visual Studio Code, all the way to being committed in a GitHub repository, and it turns out that <a href="https://www.microsoft.com/en-us/Investor/events/FY-2023/Morgan-Stanley-TMT-Conference#:~:text=Scott%20Guthrie%3A%20I%20think%20you%27re,is%20now%20AI%2Dgenerated%20and%20unmodified">at least 40% of the code generated is committed unmodified.</a></p>
<p>That was a year ago. That ratio is almost certainly higher today. <em>(Setback.)</em></p>
<p>Now we have Apple’s shenanigans with Progressive Web Apps in the EU and, no matter whether you’re gullible enough to buy Apple’s line or whether you’re a rabid everything-Apple-sucks hater who is incapable of seeing the good that Apple occasionally does, losing such an important capability <a href="https://www.thisdot.co/blog/the-renaissance-of-pwas">only a few months after they first managed to make it truly usable</a> is, well, <em>a setback</em>. A dispiriting one at that.</p>
<p>Between the obvious cycle triggered by the lay-offs and the generally inhospitable environment that the software industry has evolved into, it’s hard to feel excited or motivated by software development.</p>
<p>And we do need to feel motivated, at least in part. The word “passion” is almost always a red flag for any given industry, but software development is a <em>creative</em> business. You may all go around cosplaying engineers, but your average coder is about as close to being an engineer as a fan cosplaying as Hugh Laurie playing House is to being a medical doctor.</p>
<p><em>Software is a creative industry with more in common with media production industries than housebuilding.</em></p>
<p>As such, a baseline intrinsically-motivated curiosity about the form is one of the most powerful assets you can have when doing your job. It helps you solve problems and come up with new ideas.</p>
<p>The current cycle of decline and disinvestment in people makes that hard – probably next to impossible – but we can try to find our own ways forward where we can.</p>
<hr>
<p>The weekend before I wrote this, I was feeling lost – unsure about my place and future in the industry.</p>
<p>A walk helped. Doing something creative like photographing what I saw around town helped as well.</p>
<p>But what reminded me of how much <em>fun</em> you can have with software was having a go at porting this excellent implementation of the <a href="https://github.com/andrewstephens75/as-dithered-image">Atkinson Dithering algorithm</a> to use the <a href="https://sharp.pixelplumbing.com/"><code>sharp</code></a> node.js library and then playing around with processing and laying my photos into what you see in this essay. The image at the beginning is of the one wall that remains of the old wool factory by the waterfall in Varmá.</p>
<p><em>Villikettir Suðurlands</em> specialise in rescuing cats, both feral and abandoned housecats, here in the south of Iceland. The picture below is of the house where they keep some of the cats waiting for somebody to adopt them. They’re in from the cold.</p>
<p>Not everything is a setback.</p>
<img src="https://www.baldurbjarnason.com/img/hveragerdi-7-1-dithered.png" alt="A high key, heavily dithered image of a small house covered with corrugated iron. In the window there are two cats loafing.">
