title: What can a software developer do about climate change?
url: https://codewithoutrules.com/2019/09/10/software-developers-climage-change/
hash_url: c982d6d5411add24cca0c698ba457826

<p>Pines and firs are dying across the Pacific Northwest, fires rage across the Amazon, it&rsquo;s the hottest it&rsquo;s ever been in Paris—climate change is impacting the whole planet, and things are not getting any better.
<strong>You want to do something about climate change, but you&rsquo;re not sure what.</strong></p>
<p>If you do some research you might encounter an essay by Bret Victor—<a href="http://worrydream.com/ClimateChange/">What can a technologist do about climate change?</a>
There&rsquo;s a whole pile of good ideas in there, and it&rsquo;s worth reading, but the short version is that you can use technology to &ldquo;create options for policy-makers.&rdquo;</p>
<p>Thing is, policy-makers aren&rsquo;t doing very much.</p>
<p><strong>So this essay isn&rsquo;t about technology, because technology isn&rsquo;t the bottleneck right now, it&rsquo;s about policy and politics what you can do about it.</strong>
It&rsquo;s still written for software developers, because that&rsquo;s who I write for, but also because software developers often have access to two critical catalysts for political change.
And it&rsquo;s written for software developers in the US, because that&rsquo;s where I live, and because the US is a big part of the problem.</p>
<p>But before I go into what you can do, let me tell you the story of a small success I happened to be involved in, a small step towards a better future.</p>

<h2>Infrastructure and the status quo</h2>
<p>About a year ago I spent some of my mornings handing out pamphlets to bicycle riders.
I looked like an idiot: in order to show I was one of them I wore my bike helmet, which is weirdly shaped and the color of fluorescent yellow snot.</p>
<p>After finding an intersection with plenty of bicycle riders and a long red light that forces them to stop, I would do the following:</p>
<ol>
<li>When the light turns red, step into the street and hand out the pamphlet.</li>
<li>Keep an eye out for the light changing to green so that I didn&rsquo;t get run over by moving cars.</li>
<li>Twiddle my thumbs waiting for the next light cycle.</li>
</ol>
<p>It was boring, and not very glamorous.</p>
<p>I was one of just many volunteers, and besides gathering signatures we also held rallies, had conversations with city councilors and staff, wrote emails, talked at city council meetings—it was a process.
The total effort took a couple of years (and I only joined in towards the end)—but in the end we succeeded.</p>
<p><strong>We succeeded in having the council pass a short ordinance, a city-level law in the city of Cambridge, Massachusetts.</strong>
The ordinance states that whenever a road that was supposed to have protected bike lanes (per the city&rsquo;s Bike Plan) was rebuilt from scratch, it would have those lanes built by default.</p>
<p>Now, clearly this ordinance isn&rsquo;t going to solve climate change.
In fact, <em>nothing</em> Cambridge does as a city will solve climate change, because there&rsquo;s only so much impact 100,000 people can have on greenhouse gas emissions.</p>
<p><strong>But while in some ways this ordinance was a tiny victory in a massive war, if we take a step back it&rsquo;s actually more important than it seems.</strong>
In particular, this ordinance has three effects:</p>
<ol>
<li>Locally, safer bike infrastructure means more bicycle riders, and fewer car drivers. That reduces emissions—a little.</li>
<li>Over time, more bicycle riders can kick off a positive feedback cycle, reducing emissions even more.</li>
<li>Most significantly, local initiatives spread to other cities—kicking off these three effects in those other cities.</li>
</ol>
<p>Let&rsquo;s examine these effects one by one.</p>
<h2>Effect #1: Fewer cars, less emissions</h2>
<p>About 43% of the greenhouse gas emissions in Massachusetts are due to transportation; for the US overall it&rsquo;s 29% (<a href="https://www.mass.gov/service-details/ma-ghg-emission-trends">ref</a>).
And that means cars.</p>
<p><strong>The reason people in the US mostly drive cars is because all the transportation infrastructure is built for cars.</strong>
No bike lanes, infrequent, slow and non-existent buses, no trains&hellip;
Even in cities, where other means of transportation are feasible, the whole built infrastructure sends the very strong message that cars are the only reasonable way to get around.</p>
<p>If we focus on bicycles, our example at hand, the problem is that riding a bicycle can be dangerous—mostly because of all those cars!
But if you get rid of the danger and build good infrastructure—dedicated protected bike lanes that separate bicycle riders from those dangerous cars—then bicycle use goes up.</p>
<p>Consider what Copenhagen achieved between 2008 and 2017 (<a href="https://kk.sites.itera.dk/apps/kk_pub2/pdf/1962_fe6a68275526.pdf">ref</a>):</p>
<table><thead>
<tr>
<th></th>
<th>2008</th>
<th>2018</th>
</tr>
</thead><tbody>
<tr>
<td># of seriously injured cyclists</td>
<td>121</td>
<td>81</td>
</tr>
<tr>
<td>% who residents who feel secure cycling</td>
<td>51</td>
<td>77</td>
</tr>
<tr>
<td>% who cycle to work/school</td>
<td>37</td>
<td>49</td>
</tr>
</tbody></table>
<p><strong>With safer infrastructure for bicycles, perception of safety goes up, and people bike more and drive less.</strong>
Similarly, if you have frequent, fast, and reliable buses and trains, people drive less.
And that means less carbon emissions.</p>
<p>In Copenhagen the number of kilometers driven by cars was flat or slightly down over those 10 years—whereas in the US, it&rsquo;s up 6-7% (<a href="https://fred.stlouisfed.org/series/M12MTVUSM227NFWA">ref</a>).</p>
<h2>Effect #2: A positive feedback loop</h2>
<p>The changes in Copenhagen are a result of a plan the city government there adopted in 2011 (<a href="https://en.wikipedia.org/wiki/Cycling_in_Copenhagen#Current_bicycle_strategy_(2011%E2%80%932025)">ref</a>): they&rsquo;re the result of a policy action.
And the political will was there in part because there were <em>already</em> a huge number of bicycle riders.
So it&rsquo;s a positive feedback loop, and a good one.</p>
<p>Let&rsquo;s see how this is happening in Cambridge:</p>
<ul>
<li>Cambridge has a slowly growing number of bicycle rider.
This means more political support for bike infrastructure—if there&rsquo;s a group that can mobilize that support!</li>
<li>With the ordinance, more roads will have safe infrastructure.
For example, one neighborhood previously had a safe route only in one direction; the other direction will be rebuilt with a protected bike lane in 2020.</li>
<li>With safer infrastructure, there will be more bicycle riders, and therefore more support by residents for safer infrastructure.
Merely having support isn&rsquo;t enough, of course, and I&rsquo;ll get back to that later on.</li>
</ul>
<p>If Copenhagen can reach 50% of residents with a bicycle commute, so can Cambridge—and the ordinance is a good step in that direction.</p>
<h2>Effect #3: The idea spreads</h2>
<p>The Cambridge ordinance passed in April 2019—and the idea is spreading elsewhere:</p>
<ul>
<li>The California State Assembly is voting on a law with similar provisions (<a href="https://www.calbike.org/our_initiatives/streets_for_everyone/complete_streets/">ref</a>), through a parallel push by Calbike.</li>
<li>In May 2019 a Washington DC Council member introduced a bill which among other points has the same rebuild requirements as the Cambridge ordinance (<a href="https://usa.streetsblog.org/2019/05/08/d-c-bill-would-make-protected-bike-lanes-mandatory/">ref</a>).</li>
<li>The Seattle City Council passed an ordinance, parts of which were literally copy/pasted from the Cambridge ordinance (<a href="https://www.seattletimes.com/seattle-news/transportation/seattle-city-council-approves-new-bike-lane-requirements-calls-for-more-bike-lane-funding/">ref</a>).</li>
</ul>
<p>All of this is the result of local advocacy—but I&rsquo;ve no doubt Cambridge&rsquo;s example helped.
It&rsquo;s always easier to be the second adopter.
And the examples from these larger localities will no doubt inspire other groups and cities, spreading the idea even more.</p>
<h2>Change requires politics</h2>
<p>Bike infrastructure is just an example, not a solution—but there are three takeaways from this story that I&rsquo;d like to emphasize:</p>
<ul>
<li>If you want to change policy, you need to engage in politics.</li>
<li>Politics are easier to impact on the local level.</li>
<li>Local policy changes have a cumulative, larger-scale impact.</li>
</ul>
<p>By politics I don&rsquo;t just mean having an opinion or voting for a candidate, but rather engaging in the process of how policy decisions are made.</p>
<p><strong>Merely having an opinion doesn&rsquo;t change anything.</strong>
For example, two-thirds of Cambridge residents support building more protected bike lanes (<a href="https://www.cambridgebikesafety.org/2019/03/19/two-thirds-cambridge-residents-protected-want-protected-bike-lanes/">ref</a>).
But that doesn&rsquo;t mean that many protected lanes are getting built—the neighboring much smaller city of Somerville is building far more than Cambridge.</p>
<p>The only reason the city polled residents about bike lanes is because, one suspects, all the fuss we&rsquo;d been making—emails, rallies, meetings, city council policy orders—made the city staff wonder if bike infrastructure really had a lot of public support or not.</p>
<p><strong>Voting results in some change, but not enough.</strong>
Elected officials and government staff have lots and lots of things to worry about—if they&rsquo;re not being pressured to focus on a particular issue, it&rsquo;s likely to fall behind.</p>
<p>What&rsquo;s more, the candidates you get to vote for have to get on the ballot, and to do that they need money (for advertising, hiring staff, buying supplies).
Lacking money, they need volunteer time.</p>
<p>And it&rsquo;s much easier for a small group of rich people to provide that support to the candidates <em>they</em> want—so by the time you&rsquo;re voting, you only get to choose between candidates that have been pre-vetted (I highly recommend reading <em><a href="https://www.press.uchicago.edu/ucp/books/book/chicago/G/bo3624792.html">The Golden Rule</a></em> to understand how this works on a national level).</p>
<h2>What you can do: Become an activist</h2>
<p><strong>In the end power is social.</strong>
Power comes from people showing up to meetings, people showing up for rallies, people going door-to-door convincing other people to vote for the right person or support the right initiative, people blocking roads and making a fuss.</p>
<p>And that takes time and money.</p>
<p>So if you want to change policy, you need to engage in politics, with time and money:</p>
<ul>
<li>You can volunteer for candidates&rsquo; political campaigns, as early as possible in the process.
Too many good candidates get filtered out before they even make the ballot.
That doesn&rsquo;t mean you can just go home after the election—that&rsquo;s when the real work of legislation starts, which means activism is just as important.</li>
<li>You can volunteer with groups either acting on a particular issue (transportation, housing policy) or more broadly on climate change.</li>
<li>Also useful is donating money to political campaigns, both candidates and issue-based organizations.</li>
</ul>
<p>Here are some policies you might be interested in:</p>
<ul>
<li>Transportation policy determines what infrastructure is built—and the current infrastructure favors privately-owned cars over public transportation and bicycles.</li>
<li>Zoning laws determine what gets built and where.
Denser construction would reduce the need for long trips, and more efficient buildings (ideally net zero carbon) would reduce emissions from heating and cooling.</li>
<li>Moving utilities from private to public ownership, so they can focus on the public good and not on profit.</li>
<li>Bulk municipal contracts for electricity: this allows for cheaper electricity for all residents, and to have green energy as the default.</li>
<li>State-level carbon restrictions or taxes.</li>
</ul>
<h2>Where you should do it: Start local</h2>
<p>If you are going to become an activist, the local level is a good starting point.</p>
<ul>
<li><strong>An easier first step:</strong> Cambridge has 100,000 residents—city councilors are routinely elected with just 2500 votes.
That means impacting policies here is much easier than at a larger scale.
Not only does this mean faster results, it also means you&rsquo;re less likely to get discouraged and give up—you can see the change happening.</li>
<li><strong>Direct impact:</strong> A significant amount of greenhouse gas emissions in the US are due to causes that are under control of local governments.</li>
<li><strong>Wider impact:</strong> As in the case of Cambridge&rsquo;s ordinance, local changes can be adopted elsewhere.</li>
</ul>
<p>Of course, local organizing is just the starting point for creating change on the global level.
But you have to start somewhere.
And global change is a lot easier if you have thousands of local organizations supporting it.</p>
<h2>It&rsquo;s a good to be a software developer</h2>
<p><strong>Let&rsquo;s get back to our starting point—you&rsquo;re paid to write software, you want to do something about climate change.</strong>
As a software developer you likely have access to the inputs needed to make political campaigns succeed—both candidate-based and issue-based:</p>
<ul>
<li><strong>Money:</strong> Software developers tend to get paid pretty well, certainly better than most Americans.
Chances are you have some money to spare for political donations.</li>
<li><strong>Time:</strong> This one is a bit more controversial, but in my experience many programmers can get more free time if they want to.</li>
</ul>
<p><strong>If you don&rsquo;t have children or other responsibilities, you can work a 40-hour workweek, leaving you time for other things.</strong>
Before I got married I worked full-time and went to a local adult education college half-time in the evenings: it was a lot of work, but it was totally doable.
<a href="/2019/04/03/setting-boundaries-at-work/">Set boundaries at your job</a>, and you&rsquo;ll have at least some free time for activism.</p>
<p><strong>You can also negotiate a shorter workweek, which is possible in part because software developers are in such demand.</strong>
I&rsquo;ve done this, I&rsquo;ve <a href="/2018/01/08/part-time-programmer/">interviewed people who have done it</a>, I&rsquo;ve found <a href="/2019/05/09/part-time-software-developer/">many random people on the Internet who have done it</a>—it is possible.</p>
<p>If you need help doing it yourself, I&rsquo;ve written <a href="/3dayweekend/">a book to help you negotiate a shorter workweek</a>.
<strong>If you want to negotiate a shorter workweek so you have time for political activism, you can use the code <code>FIGHTCLIMATECHANGE</code> to <a href="/3dayweekend/">get the book</a> for 60% off.</strong></p>
<h2>Some common responses</h2>
<h3>&ldquo;There will never be the political will to make this happen&rdquo;</h3>
<p>Things do change, for better and for worse, and sometimes unexpectedly.
To give a couple of examples:</p>
<ul>
<li>In Ireland, the Catholic Church went from all-powerful to losing badly, most recently with Ireland legalizing abortion.</li>
<li>The anti-gay-marriage <em>Defense of Marriage Act</em> was passed by veto-proof majorities of Congress in 1996—and eight years later in 2004 the first legal gay marriage took place right here in Cambridge, MA.</li>
</ul>
<p>The timelines for <a href="https://en.wikipedia.org/wiki/Timeline_of_same-sex_marriage_in_the_United_States">gay marriage</a> and <a href="https://en.wikipedia.org/wiki/Timeline_of_cannabis_laws_in_the_United_States">cannabis legalization</a> in the US are illuminating: these things didn&rsquo;t just happen, it was the result of long, sustained activist efforts, much of it at the local level.</p>
<p>Local changes do make a difference.</p>
<h3>&ldquo;Politics is awful and broken&rdquo;</h3>
<p>So are all our software tools, and somehow we manage to get things done!</p>
<h3>&ldquo;I don&rsquo;t like your policy suggestions, we should do X instead&rdquo;</h3>
<p>No problem, find the local groups that promote your favorite policies and join them.</p>
<h3>&ldquo;The necessary policies will never work because of problem Y&rdquo;</h3>
<p>Same answer: join and help the local groups working on Y.</p>
<h3>&ldquo;It&rsquo;s too late, the planet is doomed no matter what we do&rdquo;</h3>
<p>Perhaps, but it&rsquo;s very hard to say.
So we&rsquo;re in Pascal&rsquo;s Wager territory here: given even a tiny chance there is something we can do, we had better do our best to make it happen.</p>
<p>And even if humanity really is doomed, there&rsquo;s always the hope that someday a hyperintelligent species of cockroach will inherit the Earth.
And when cockroach archaeologists try to reconstruct our history, I would like them to be able to say, loosely translated from their complex pheromone-and-dancing system of communication: &ldquo;These meatsacks may not have been as good at surviving as us cockroaches—but at least they tried!&rdquo;</p>
<h2>Time to get started</h2>
<p>If you find this argument compelling—that policy is driven by power, and that power requires social mobilization—then it&rsquo;s up to you to take the next step.
Find a local group or candidate pushing for a policy you care about, and show up for the next meeting.</p>
<p>And the meeting after that.</p>
<p>And then go to the rally.</p>
<p>And knock on doors.</p>
<p>And make some friends, and make some changes happen.</p>
<p>Some of the work is fun, some of it is boring, but there&rsquo;s plenty to do—time to get started!</p>
