title: Sympathy vs. Empathy in UX
url: https://www.nngroup.com/articles/sympathy-vs-empathy-ux/
hash_url: eae184de757f4384212d32a1273772cd

<p>As UX professionals, it is our job to advocate on behalf of our users. To do it, we must understand them. Understanding our users means building empathy for human beings who experience the product or service we create. </p>

<p>In an effort to practice empathy, many teams mistakenly practice sympathy. Although these words have different meanings, teams often incorrectly use sympathy and empathy interchangeably. This confusion results in a large gap in their understanding and in an inability to address real human needs. </p>

<p>The goal of this article is to help you reflect on your current UX practice — are you unknowingly practicing pity and sympathy instead of empathy?</p>

<h2>What Is Sympathy?</h2>

<p>Definition: <strong>Sympathy</strong> is the acknowledgement of the suffering of others.</p>

<p>Sympathy is often the reaction (in the form of sorrow or pity) to the hardship or plight of another person. However, in contrast to empathy, there’s still a feeling of distance between you and the other person, and their adversity is not something you personally relate to or expect to share. You don’t envision yourself suffering the user's problems (and you definitely don't share those problems now), much as you do recognize that some users have problems.</p>

<p>In UX, sympathy is limited to acknowledging that users are going through a difficult scenario, task, or journey. If we are sympathetic to our users, it does not mean that we put ourselves in their shoes and feel their pain or annoyance. For example, when we’re designing an accessible website for people who are blind, we may express sympathy by acknowledging their potential challenges: </p>

<ul>
	<li>“It will be hard to consume the content if you can’t see the infographics.”</li>
	<li>“This font is kind of small and light. It may be hard for someone older to read.”</li>
	<li>“It would be tough to navigate this website with a screen reader.” </li>
</ul>

<p>It is true that some sympathy is better than none. For example, it is better to have sympathy for users with <a href="https://www.nngroup.com/articles/computer-skill-levels/">limited technical skills</a> than despise them for their shortcomings. However, <strong>the true goal of design is not to be nice to users, but to empower them</strong>. For example, this is why we don’t recommend lengthy error messages about how sorry we are about an error. Rather, we suggest that <a href="https://www.nngroup.com/articles/errors-forms-design-guidelines/">error messages allow users to quickly correct the problem</a> and move on.</p>

<h2>What Is Empathy? </h2>

<p>Empathy is a step beyond sympathy, and more complex in nature. </p>

<p>Definition: <strong>Empathy</strong> is the ability to fully understand, mirror, then share another person’s expressions, needs, and motivations. </p>

<p>In UX, empathy enables us to understand not only our users’ immediate frustrations, but also their hopes, fears, abilities, limitations, reasoning, and goals. It allows us to dig deep into our understanding of the user and create solutions that will not only solve a need, but effectively improve our users’ lives by removing unnecessary pain or friction. Instead of just designing an accessible website, practicing empathy is using a screen reader, blindfolded, in order to complete a task on your own website. </p>

<ul>
	<li>“I am struggling to find my way around the site.”</li>
	<li>“This is much tougher than I thought it would be.”</li>
	<li>“I will advocate for the changes that need to be made.”</li>
</ul>

<h2>The Spectrum of Empathy </h2>

<p>There is no firm threshold that marks one’s transition from sympathy to empathy. Rather, the relation between the two is best represented on a spectrum with pity (the most disconnected and abstracted version of sympathy) on one end and compassion (the more connected and embodied version of empathy) on the other. </p>

<p><img alt="The Spectrum of Empathy by Sarah Gibbons" src="https://media.nngroup.com/media/editor/2019/03/22/screen-shot-2019-03-22-at-90954-am.png"/></p>

<p><em>The spectrum of empathy includes pity, sympathy, empathy, and compassion. Pity and sympathy require little to no effort or understanding, while empathy and compassion require effort to understand and engagement to produce a positive change. </em></p>

<p> </p>

<p>Pity is simply when you feel sorry for somebody else. You don’t like their unfortunate situation, and maybe you’ll even do something to rectify the situation, but mostly to make your own unpleasant feelings go away.</p>

<p>On the other end of the spectrum is compassion, the feeling where you relate the most to the users as independent actors, as opposed to objects. ("Actors" in this context means that we recognize that the users have their own purposes, wants, and needs, and that they are acting to satisfy what <em>they </em>want to accomplish, not what we think they ought to do or want. Thus, we don't impose our priorities or preferences upon the users, which would be an act of objectifying them that's more characteristic of sympathy.) Compassion is a call to action derived from empathy — when our understanding of another’s thoughts or feelings give us the compulsion, duty, or desire to help change that person’s situation for the better.  </p>

<h2>How to Practice Empathy in UX </h2>

<h4> </h4>

<ul>
	<li>
	<h4>Use Qualitative Research Methods  </h4>
	</li>
</ul>

<p>Practicing empathy in UX must begin with user research. We must set aside ego and assumptions and immerse ourselves in research. Qualitative methods, such as user interviews, cognitive mapping, and <a href="https://www.nngroup.com/articles/diary-studies/">diary studies</a>, allow us to dig into user behaviors, motivations, and concerns. </p>

<p>Remember to use <a href="https://www.nngroup.com/articles/open-ended-questions/">open-ended questions</a>. When you ask users to explain things to you, they often reveal surprising <a href="https://www.nngroup.com/articles/mental-models/">mental models</a>, problem-solving strategies, hopes, and fears. For example: </p>

<p><em>What makes you happy? </em>instead of <em>Are you happy? </em></p>

<p><em>How has your family affected you? </em>instead of <em>Are you close to your family?</em></p>

<p><em>What would make you stronger? </em>instead of <em>Tell me your weaknesses.</em></p>

<p>Practice empathy as you conduct research. Be aware that you don’t know what people are going through and what will trigger a memory or be difficult for them. </p>

<h4> </h4>



<p>Make accessibility a part of your research plan. This approach allows you to test your assumptions and explore potential opportunities for improvement with actual end users. Use <a href="https://www.afb.org/default.aspx">well-known organizations</a>, <a href="https://nfb.org/about-us/state-affiliates">state chapters</a>, or <a href="https://carroll.org/">local training centers</a> to help you recruit participants with disabilities.</p>

<h4> </h4>

<ul>
	<li>
	<h4>Have Your Team Watch Research Sessions and See Real Users</h4>
	</li>
</ul>

<p>When conducting research, invite all team members, and key stakeholders to observe the sessions. <a href="https://www.nngroup.com/articles/stakeholders-and-user-testing/">Doing this vastly increases the potential of empathy</a>, and the corresponding acceptance of research findings. Seeing is believing. And seeing and interacting with the user live is even more powerful.</p>

<p>Prior to inviting your coworkers to user testing, you may need to spend some time <a href="https://www.nngroup.com/articles/collaborating-stakeholders/">evangelizing UX research</a> within your team or organization. Make sure everyone knows how deep user-experience design needs to be and what you can accomplish: saved time, minimal rework, and a product that solves <a href="https://www.nngroup.com/articles/user-need-statements/">a real user need</a>.</p>

<p>Make sure that you post recordings from user sessions in a place where people can access them and watch them on their own, in case they cannot observe live.</p>

<h4> </h4>

<ul>
	<li>
	<h4>Use Videos of Users Whenever Presenting Research Findings to Stakeholders</h4>
	</li>
</ul>

<p>Supplement your findings and recommendations with video clips showing how users actually perform that task. Not only will your findings be more compelling, but you’ll also be building general empathy towards your audience. Make sure you show a variety of people in your videos (it shouldn’t be hard if you followed our first guideline and recruited a diverse participant group) — with different backgrounds, different demographics, and different abilities.</p>

<h4> </h4>



<p><a href="https://www.nngroup.com/articles/empathy-mapping/">Empathy maps</a> capture users’ emotions, hopes, and fears and distill your knowledge of the users into one place. An empathy map can help you discover gaps in your current knowledge and identify the types of research needed to address it.</p>

<p>Even more importantly, an empathy map can help others become empathic towards users because it can act as a source of truth throughout a project and protect it from bias or unfounded assumptions. Empathy is a complex skill, and empathy maps reduce the risk of misalignment because everyone has the same visual baseline. </p>

<h4> </h4>



<p>“You are not the user” may sound like a cliché, but it’s part of the human psychology to think <a href="https://www.nngroup.com/articles/false-consensus/">that others think and behave in the same way that we do</a>. If everybody in your team is male, younger than 30, and with a tech background, you’ll end up with designs that will implicitly favor that user group.</p>

<p>Recruit team members with a variety of backgrounds and demographics. That will not guarantee empathy for users, but will at least be a first step in the right direction. Diversity must go wide to include experiences, skills, and attitudes acquired and evolved over each employee’s lifetime — known as ‘acquired diversity.’ In broad terms, as defined by Chloe Heath, acquired diversity is a person’s experience of the world and the cultural quirks accumulated over a long period. </p>

<h4> </h4>

<ul>
	<li>
	<h4>Build Empathy into Your Design Guidelines</h4>
	</li>
</ul>

<p>Within this diverse team, create protocols that encourage empathy. For example, Caroline Jarrett’s<a href="https://www.uxmatters.com/mt/archives/2010/06/the-question-protocol-how-to-make-sure-every-form-field-is-necessary.php"> Question Protocol</a> uses consistent intention and prioritization behind each and every question, rather than asking all possible questions to the user (some of which may make the user feel inadequate or uncomfortable, bring up bad memories, etc.).</p>

<p>If you’ve been with your team for a while, you may have a good idea of the types of erroneous assumptions it tends to make. Create specific guidelines that can act as a check point against your team’s bad habits, especially when it comes to empathy. For example, if your team is prone to not changing designs that don’t work for specific users, consider introducing a guideline that can correct this behavior. The guideline could be <em>Each design must be tested with a diverse user group that maps onto our target demographics</em>. Instead of <a href="https://www.nngroup.com/articles/no-validate-in-ux/"><em>Let’s validate this design, say Let’s learn what works and what doesn’t work well for users and why</em></a>.</p>

<h2>Conclusion </h2>

<p>Empathy in UX is essential. It is a bridge into our user’s minds and our greatest asset as UX professionals. Empathy allows us to design with intent, introduce focus and clarity, advocate on behalf of our users, and challenge our assumptions.</p>

<p>(For more on the difference between sympathy and empathy, see also Neel Burton's <a href="https://www.psychologytoday.com/us/blog/hide-and-seek/201505/empathy-vs-sympathy">Empathy vs Sympathy</a>.)</p>
